% dri_batch.m
%
% Segementation of embryo images in a directory
% 
% Qi Li
% 10/08/08
% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% data path should be properly setup
path_im = 'C:\workspaces\data\BDGP_2\8\';
path_contour = 'C:\workspaces\data\BDGP_2\contour_8\';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters
scale_factor = 0.4;             % scaling image
threshold_nonexpressive = 0.1;  
threshold_align = 10;           % gap threshold used in align_edgels; 

load trainedLDA.mat;    % load "G" and "median_value"
load models.mat         % load "models" used to rectify a contour
load eigenshape.mat     % load "G_eigen" and "contour_mean"

dim = 150;   % dimension of eigen-space

w_elastics = 0.005;  
w_smooth = 0.005;
w_im_std = 0.49;
w_im_canny = 0.5;  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get image filenames in the directory
path_working = pwd;
cmd_to_dataset = sprintf('%s%s', 'cd ', path_im);
eval(cmd_to_dataset);
filename_all = ls;
cmd_back_to_work = sprintf('%s%s', 'cd ', path_working);
eval(cmd_back_to_work);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% batch processing
results = [];
for idx_file = 4 : length(filename_all);

  idx_file

  contourname = sprintf('%s%s%s', path_contour, filename_all(idx_file,:), '.dat');
  contour_groundtruth = load(contourname); 
  contour_groundtruth = contour_groundtruth * scale_factor;


  imname_in = sprintf('%s%s', path_im, filename_all(idx_file,:))
  %filename_out = sprintf('%s%d%s', path_out, idx_file, '.png');

  im1 = imread(imname_in);

  im1 = imresize(im1, scale_factor);
  im_rgb = double(im1);
  im = im_rgb(:,:,1);

  % get edgels
  edge_type = 'canny';
  im_rgb = im1;
  get_edges;  % pre-processing

  % remove expressive pixels
  labels_internal =  remove_expressive_pixels(edgels,size(edgels,1), im_rgb(:,:,1), im_rgb(:,:,2), im_rgb(:,:,3), ...
			    size(im_rgb,1), size(im_rgb,2), G, median_value);
  labels_internal = floor(labels_internal);
  idx_inliers = find(labels_internal == 1);
  edgels = edgels(idx_inliers, :);
  im = rgb2gray(im_rgb); 
  figure; imagesc(im); colormap(gray); axis off; hold on; 
  plot(edgels(:,1), edgels(:,2), 'k+', 'LineWidth', 4, 'MarkerSize', 6);  


  % remove nonexpresive outliers
  labels_internal =  remove_nonexpressive_outliers(edgels,size(edgels,1), im_rgb(:,:,1), im_rgb(:,:,2), im_rgb(:,:,3), ...
			    size(im_rgb,1), size(im_rgb,2), threshold_nonexpressive);
  labels_internal = floor(labels_internal);
  idx_inliers = find(labels_internal == 1);
  edgels_inliers = edgels(idx_inliers, :);
  edgels = edgels_inliers;


  im = zeros(size(im,1), size(im,2))*255;
  im(1,1)=255;
  figure; imagesc(im); colormap(gray); axis off; hold on; 
  %plot(edgels(:,1), edgels(:,2), 'k+', 'LineWidth', 4, 'MarkerSize', 6);  
  plot(edgels(:,1), edgels(:,2), 'w+', 'LineWidth', 1, 'MarkerSize', 1, 'Color', [1,1,1]);  


  centroid = mean(edgels, 1); 
  edgels_centered = edgels - ones(length(edgels),1) * centroid; 
  contour = align_edgels(edgels_centered, length(edgels), threshold_align); 
  idx = find(contour(:,1) ~= 0); 
  contour = contour(idx, :); 

  % remove outliers via ransac
  edgels = contour; 
  remove_via_ransac;
  contour = inliers_best; 

  % eigen analysis 
  contour_eigen =  eigen_reconstruct(contour, G_eigen, dim, contour_mean);
  contour_eigen = contour_eigen + ones(360,1) * centroid;

  % measure 
  contour_out = contour_eigen;
  disp = measure_contours_displacement(contour_out, length(contour_out), ...
                          contour_groundtruth, length(contour_groundtruth))

  figure; imagesc(im1); colormap(gray); hold on;
  plot(contour_out(:,1), contour_out(:,2), 'r-', 'LineWidth', 3); hold on;

  results = [results, disp];

  pause

end % end of dri_batch
