function contour_reconstruct = eigen_reconstruct(test, G_eigen, dim, contour_mean);
% eigen_reconstruct.m
% 
% Input:  G_eigen ---Eigen contour space 
%         test    ---A contour (un-normalized)
%
% Output: contour_reconstruct---A contour reconstructed by eigen space
% 
% Qi Li
% 08/25/08
%


G_eigen = G_eigen(:,1:dim);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% interpolate contour
n = 20;                        % interpolation parameter
test_working = interpolate_contour(test, n);
len = length(test_working);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% normalize and align the contour
centroid = mean(test_working, 1);
test_working = test_working - ones(len,1) * centroid;           % centeralized, center = (0,0);
[test_working, rotation] = normalize_orientation(test_working); % normalize orientation
test_aligned = align_contour(test_working, len);                % order by angular theta


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% difference from mean contour
test_aligned_var = test_aligned - contour_mean; 
test_aligned_var = test_aligned_var(:);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reconstruction
contour_reconstruct = test_aligned_var' * G_eigen * G_eigen';                         % eigen reconstruction
contour_reconstruct = reshape(contour_reconstruct, 360,2) + contour_mean; % recover mean shape
contour_reconstruct = contour_reconstruct * rotation;                     % recover orientation
contour_reconstruct = contour_reconstruct + ones(360, 1) * centroid;      % recover location

%figure; plot(test_aligned(:,1), test_aligned(:,2), '-'); hold on;
%plot(test(:,1), test(:,2), '-'); hold on;
%plot(contour_reconstruct(:,1), contour_reconstruct(:,2), 'r-');
