% fitting_mixed_guassian
%
% p(x) = sum_i w(i) *e^(-(x-c(i))^2/sigma(i)^2)
%
% input: points
% output: w, sigma, x0
%
% 12/25/07

if 0 == 1  % simulated data
pts = zeros(256,2);
centroids = [100 180];
sigma = [30 30];
w = [0.3 0.7];

pts(:,1) = (1:256)';
g1 = exp(-(pts(:,1) -centroids(1)).^2/sigma(1)^2) + rand(256,1)*0.1;
g2 = exp(-(pts(:,1) -centroids(2)).^2/sigma(2)^2) + rand(256,1)*0.1;
pts(:,2) = w(1)*g1 + w(2)*g2;
plot(pts(:,2));

else % real data

pts = zeros(256,2);
pts(:,1) = (1:256)';
histogram = histogram /(320*240);
pts(:,2) = histogram(1,:)'+histogram(3,:)';

end 

estimate = zeros(256,1);

num_cluster = 2;

centroids = [70 200];
sigma = [10 10];
[Y, I] = max(pts(:,2));
regularization = 1e-7;

pts(:,2) = pts(:,2) + regularization; 

num = 256;
const = zeros(num,2);
method = 2;

%while residue_current > threshold
for loop = 1:30

   for cluster_id = 1: num_cluster

   % update sigma and weight (w) by the updated cluster
   idx_inside = find(abs(pts(:,1) - centroids(cluster_id)) < 1*sigma(cluster_id));

 if 0 == 1
   w(cluster_id) = max(pts(idx_inside,2));
%   w = max(pts(idx_inside,2))-regularization;
 end

   % compute w by linear fitting 
   c1 = exp(-(pts(:,1)-centroids(1)).^2./sigma(1)^2);
   c2 = exp(-(pts(:,1)-centroids(2)).^2./sigma(2)^2);
   A = [c1 c2 -pts(:,2)];
   correlation = A' * A;
   [U S V] = svd(correlation);
   w = U(1:2,3) / U(3,3);
   w = abs(w);
   w = w / sum(w);

   w_ln = log(w(cluster_id));

   % update sigma(cluster_id)
   switch method
   case 1, 
     sigma(cluster_id) = std(pts(idx_inside,1))  % may not be precise
   case 2, 
%     y_ln = log(pts(:,2));
     other_cluster_id = 3 - cluster_id;
     y_ln = log(pts(:,2) - ... 
                w(other_cluster_id) * exp(-(pts(:,1)-centroids(other_cluster_id)).^2./sigma(other_cluster_id)^2));

     c1 = (pts(idx_inside,1) - centroids(cluster_id)).^2;
     c_t = -(w_ln - y_ln);
     c2 = c_t(idx_inside);

     A = [c1 c2];
     correlation = A' * A;
     [U S V] = svd(correlation);
     sigma(cluster_id) = sqrt(abs(U(2,2) / U(1,2)));
   end


   % update centroids
   const(:,1) = sigma(cluster_id) * sqrt(w_ln-y_ln);  % two options to update
   const(:,2) = -const(:,1);
   const = real(const);

   residue_update = zeros(2,1);
   x0_try = zeros(2,1);

   % select best option of x0
  if 0 == 0
   for i = 1: 2
%     x0_sample = pts(:,1) - const(:,i);
     x0_sample = pts(idx_inside,1) - const(idx_inside,i);
     x0_try(i) = round(median(x0_sample));

     estimate = w(cluster_id) * exp(-(pts(:,1)-x0_try(i)).^2./sigma(cluster_id)^2) + ...
                w(other_cluster_id) * exp(-(pts(:,1)-centroids(other_cluster_id)).^2./sigma(other_cluster_id)^2);
   % no difference between the following two
     residue_update(i) = norm(estimate(idx_inside) - pts(idx_inside,2));
%     residue_update(i) = norm(estimate - pts(:,2));
 
   end % end of two choices of x0
   [Y, best_one] = min(residue_update);
   centroids(cluster_id) = x0_try(best_one);

  end 

%  x0_sample = pts(idx_inside,1);
%  centroids(cluster_id) = round(median(x0_sample))

  end % end of num_clusters

end % end of loop

g1_estimate = exp(-(pts(:,1)-centroids(1)).^2./sigma(1)^2);
g2_estimate = exp(-(pts(:,1)-centroids(2)).^2./sigma(2)^2);
estimate = w(1) * g1_estimate + w(2) * g2_estimate;

figure; plot(pts(:,1), pts(:,2)-regularization, 'b+', 'LineWidth', 2, 'MarkerSize', 8); hold on;
plot(pts(:,1), g1_estimate, 'k-', 'LineWidth', 3); hold on;
plot(pts(:,1), g2_estimate, 'k-', 'LineWidth', 3); hold on;
plot(pts(:,1), estimate, 'r-', 'LineWidth', 3); 
