function [centroid, sigma] = fitting_one_guassian(data, sigma_init)
%
% p(x) = w*e^(-(x-x0)^2/sigma^2)
%
% input: points
% output: w, sigma, x0
%
% 12/25/07

if 0 == 1
channel = 2;
pts = zeros(256,2);
pts(:,1) = (1:256)';
pts(:,2) = histogram(channel,:)'/(320*240); 
end 

num = length(data);
pts = zeros(num,2);
pts(:,1) = (1:num)';

if size(data, 1) > 1 
  pts(:,2) = data;
else 
  pts(:,2) = data';
end

estimate = zeros(num,1);


[Y, x0_init] = max(pts(:,2));
regularization = 1e-5;
%regularization = Y * 0.001;

pts(:,2) = pts(:,2) + regularization; 

residue_current = 10;
residue_change = 10;
threshold = 0.001;
const = zeros(num,2);
method = 2;

sigma = sigma_init;
centroid = x0_init;


x0 = x0_init;

%while residue_current > threshold
for loop = 1:30

   % update w and sigma by the updated cluster
   idx_inside = find(abs(pts(:,1) - x0) < 1 * sigma);
  
  if length(idx_inside) == 0
     fprintf(1, 'Gaussian fitting fails; use initialized centroid and sigma\n');
     centroid = x0_init;
     sigma = sigma_init;
     break;
  end 

   w = max(pts(:,2));
%   w = max(pts(idx_inside,2));
   w_ln = log(w);
   y_ln = log(pts(:,2));
%   y_ln = log(pts(idx_inside,2));

   
   % update sigma 
   switch method
   case 1, 
     sigma = std(pts(idx_inside,1));  % may not be precise
   case 2, 
     c1 = (pts(idx_inside,1) - x0).^2;
     c_t = -(w_ln - y_ln);
     c2 = c_t(idx_inside);
     A = [c1 c2];
     correlation = A' * A;
     [U S V] = svd(correlation);
     sigma = sqrt(abs(U(2,2) / U(1,2)));
   end

   % update centroid
   const(:,1) = sigma * sqrt(w_ln-y_ln);  % two options to update
   const(:,2) = -const(:,1);
   const = real(const);

   residue_update = zeros(2,1);
   x0_try = zeros(2,1);

   % select best option of x0
   for i = 1: 2
%     x0_sample = pts(:,1) - const(:,i);
     x0_sample = pts(idx_inside,1) - const(idx_inside,i);
     x0_try(i) = round(median(x0_sample));

%     estimate(idx_inside,:) = w * exp(-(pts(idx_inside,1)-x0_try(i)).^2./sigma^2);
     estimate = w * exp(-(pts(:,1)-x0_try(i)).^2./sigma^2);

   % no difference between the following two
%     residue_update = norm(estimate(idx_inside) - pts(idx_inside,2));
     residue_update = norm(estimate - pts(:,2));
 
   end % end of two choices of x0

   [Y, best_one] = min(residue_update);

   if Y < residue_current
      residue_current = Y;
      x0 = x0_try(best_one);
      centroid = x0;
   end 

%   residue_change = abs(residue_update - residue_current);

%   figure; plot(pts(:,1), pts(:,2)-regularization, 'b+', 'LineWidth', 2, 'MarkerSize', 8); hold on;
%   plot(pts(:,1), estimate, 'k-', 'LineWidth', 3);

end % end of loop

estimate = (w-regularization) * exp(-(pts(:,1)-x0).^2./sigma^2);

%figure; plot(pts(:,1), pts(:,2)-regularization, 'k--', 'LineWidth', 3, 'MarkerSize', 8); hold on; plot(pts(:,1), estimate, 'k-', 'LineWidth', 3);
%legend('profile', 'Guassian fitting');

